Class: Net::VNC

Inherits:
Object
  • Object
show all
Defined in:
lib/net/vnc.rb,
lib/net/vnc/version.rb

Overview

the socket.

Defined Under Namespace

Classes: PointerState

Constant Summary collapse

BASE_PORT =
5900
CHALLENGE_SIZE =
16
DEFAULT_OPTIONS =
{
  :shared => false,
  :wait => 0.1
}
KEY_MAP =
YAML.load_file(keys_file).inject({}) { |h, (k, v)| h.update k.to_sym => v }
VERSION =
'1.0.1'
BUTTON_MAP =
{
  :left => 0
}

Instance Attribute Summary collapse

Class Method Summary collapse

Instance Method Summary collapse

Constructor Details

#initialize(display = ':0', options = {}) ⇒ VNC

Returns a new instance of VNC.



73
74
75
76
77
78
79
80
81
82
83
84
85
86
# File 'lib/net/vnc.rb', line 73

def initialize display=':0', options={}
  @server = 'localhost'
  if display =~ /^(.*)(:\d+)$/
    @server, display = $1, $2
  end
  @display = display[1..-1].to_i
  @options = DEFAULT_OPTIONS.merge options
  @clipboard = nil
  @pointer = PointerState.new self
  @mutex = Mutex.new
  connect
  @packet_reading_state = nil
  @packet_reading_thread = Thread.new { packet_reading_thread }
end

Instance Attribute Details

#displayObject (readonly)

Returns the value of attribute display.



71
72
73
# File 'lib/net/vnc.rb', line 71

def display
  @display
end

#optionsObject (readonly)

Returns the value of attribute options.



71
72
73
# File 'lib/net/vnc.rb', line 71

def options
  @options
end

#pointerObject (readonly)

Returns the value of attribute pointer.



71
72
73
# File 'lib/net/vnc.rb', line 71

def pointer
  @pointer
end

#serverObject (readonly)

Returns the value of attribute server.



71
72
73
# File 'lib/net/vnc.rb', line 71

def server
  @server
end

#socketObject (readonly)

Returns the value of attribute socket.



71
72
73
# File 'lib/net/vnc.rb', line 71

def socket
  @socket
end

Class Method Details

.open(display = ':0', options = {}) ⇒ Object



88
89
90
91
92
93
94
95
96
97
98
99
# File 'lib/net/vnc.rb', line 88

def self.open display=':0', options={}
  vnc = new display, options
  if block_given?
    begin
      yield vnc
    ensure
      vnc.close
    end
  else
    vnc
  end
end

Instance Method Details

#button_down(which = :left, options = {}) ⇒ Object

Raises:

  • (ArgumentError)


231
232
233
234
235
236
# File 'lib/net/vnc.rb', line 231

def button_down which=:left, options={}
  button = BUTTON_MAP[which] || which
  raise ArgumentError, 'Invalid button - %p' % which unless (0..2) === button
  pointer.button |= 1 << button
  wait options
end

#button_press(button = :left, options = {}) ⇒ Object



222
223
224
225
226
227
228
229
# File 'lib/net/vnc.rb', line 222

def button_press button=:left, options={}
  begin
    button_down button, options
    yield if block_given?
  ensure
    button_up button, options
  end
end

#button_up(which = :left, options = {}) ⇒ Object

Raises:

  • (ArgumentError)


238
239
240
241
242
243
# File 'lib/net/vnc.rb', line 238

def button_up which=:left, options={}
  button = BUTTON_MAP[which] || which
  raise ArgumentError, 'Invalid button - %p' % which unless (0..2) === button
  pointer.button &= ~(1 << button)
  wait options
end

#clipboardObject



260
261
262
263
264
265
266
267
268
269
270
271
272
273
274
# File 'lib/net/vnc.rb', line 260

def clipboard
  if block_given?
    @clipboard = nil
    yield
    60.times do
      clipboard = @mutex.synchronize { @clipboard }
      return clipboard if clipboard
      sleep 0.5
    end
    warn 'clipboard still empty after 30s'
    nil
  else
    @mutex.synchronize { @clipboard }
  end
end

#closeObject



249
250
251
252
253
254
255
256
257
258
# File 'lib/net/vnc.rb', line 249

def close
  # destroy packet reading thread

  if @packet_reading_state == :loop
    @packet_reading_state = :stop
    while @packet_reading_state
      # do nothing

    end
  end
  socket.close
end

#connectObject



105
106
107
108
109
110
111
112
113
114
115
116
117
118
119
120
121
122
123
124
125
126
127
128
129
130
131
132
133
134
135
136
137
138
139
140
141
142
143
144
145
# File 'lib/net/vnc.rb', line 105

def connect
  @socket = TCPSocket.open server, port
  unless socket.read(12) =~ /^RFB (\d{3}.\d{3})\n$/
    raise 'invalid server response'
  end
  @server_version = $1
  socket.write "RFB 003.003\n"
  data = socket.read(4)
  auth = data.to_s.unpack('N')[0]
  case auth
  when 0, nil
    raise 'connection failed'
  when 1
    # ok...

  when 2
    password = @options[:password] or raise 'Need to authenticate but no password given'
    challenge = socket.read CHALLENGE_SIZE
    response = Cipher::DES.encrypt password, challenge
    socket.write response
    ok = socket.read(4).to_s.unpack('N')[0]
    raise 'Unable to authenticate - %p' % ok unless ok == 0
  else
    raise 'Unknown authentication scheme - %d' % auth
  end

  # ClientInitialisation

  socket.write((options[:shared] ? 1 : 0).chr)

  # ServerInitialisation

  # TODO: parse this.

  socket.read(20)
  data = socket.read(4)
  # read this many bytes in chunks of 20

  size = data.to_s.unpack('N')[0]
  while size > 0
    len = [20, size].min
    # this is the hostname, and other stuff i think...

    socket.read(len)
    size -= len
  end
end

#key_down(which, options = {}) ⇒ Object



192
193
194
195
196
197
198
199
200
# File 'lib/net/vnc.rb', line 192

def key_down which, options={}
  packet = 0.chr * 8
  packet[0] = 4
  key_code = get_key_code which
  packet[4, 4] = [key_code].pack('N')
  packet[1] = 1
  socket.write packet
  wait options
end

#key_press(*args) ⇒ Object

this takes an array of keys, and successively holds each down then lifts them up in reverse order. FIXME: should wait. can’t recurse in that case.

Raises:

  • (ArgumentError)


164
165
166
167
168
169
170
171
172
173
174
175
176
177
178
# File 'lib/net/vnc.rb', line 164

def key_press(*args)
  options = Hash === args.last ? args.pop : {}
  keys = args
  raise ArgumentError, 'Must have at least one key argument' if keys.empty?
  begin
    key_down keys.first
    if keys.length == 1
      yield if block_given?
    else
      key_press(*(keys[1..-1] + [options]))
    end
  ensure
    key_up keys.first
  end
end

#key_up(which, options = {}) ⇒ Object



202
203
204
205
206
207
208
209
210
# File 'lib/net/vnc.rb', line 202

def key_up which, options={}
  packet = 0.chr * 8
  packet[0] = 4
  key_code = get_key_code which
  packet[4, 4] = [key_code].pack('N')
  packet[1] = 0
  socket.write packet
  wait options
end

#pointer_move(x, y, options = {}) ⇒ Object



212
213
214
215
216
# File 'lib/net/vnc.rb', line 212

def pointer_move x, y, options={}
  # options[:relative]

  pointer.update x, y
  wait options
end

#portObject



101
102
103
# File 'lib/net/vnc.rb', line 101

def port
  BASE_PORT + @display
end

#type(text, options = {}) ⇒ Object

this types text on the server



148
149
150
151
152
153
154
155
156
157
158
159
# File 'lib/net/vnc.rb', line 148

def type text, options={}
  packet = 0.chr * 8
  packet[0] = 4
  text.split(//).each do |char|
    packet[7] = char[0]
    packet[1] = 1
    socket.write packet
    packet[1] = 0
    socket.write packet
  end
  wait options
end

#wait(options = {}) ⇒ Object



245
246
247
# File 'lib/net/vnc.rb', line 245

def wait options={}
  sleep options[:wait] || @options[:wait]
end